﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Diagnostics;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Windows.Forms.DataVisualization.Charting;

using Mol;

namespace Examples
{
    /// <summary>
    /// 中心極限定理の確認フォーム
    /// </summary>
    public partial class FormCLT : Form
    {
        public FormCLT()
        {
            InitializeComponent();
        }

        // 各種分布関数入力パラメーター表示用（各分布毎に表示を切り替えるため）
        private TextBox[] m_Texts  = new TextBox[4];
        private Label  [] m_Labels = new Label[4];

        // 重ね合わせの回数
        private int m_sampling_size = 1;

        // 分布関数
        public enum R_METHOD
        {
            // 連続分布
            Uniform = 1,
            Gaussian,
            Exponential,
            Laplace,
            Weibull,
            Cauchy,
            Rayleigh,
            Lognormal,
            Gumbel,
            Gamma,
            Beta,
            // 離散分布
            UniformI,     // 整数区間
            UniformBits,
            UniformBits32,
            UniformBits64,
            Bernoulli,
            Geometric,
            Binomial,
            Hypergeometric,
            Poisson,
            PoissonV,
            NegBinomial
        }

        // 乱数発生器オブジェクト（リストボックス用）
        public class Gen
        {
            public RandomNumber.GENERATOR gen;
            public Gen(RandomNumber.GENERATOR g)
            {
                gen = g;
            }
            public override string ToString()
            {
                return gen.ToString();
            }
        }

        // 分布関数オブジェクト（リストボックス用）
        public class Dist
        {
            public string[] Titles;
            public R_METHOD Method;
            public Dist(R_METHOD method,string[] titles)
            {
                Method = method;
                Titles = titles;
            }
            public override string ToString()
            {
                return Titles[0];
            }
        }

        private void FormStat_Load(object sender, EventArgs e)
        {
            // パラメータ表示用ラベルとテキストボックス
            m_Texts[0] = t1;
            m_Labels[0] = l1;
            m_Texts[1] = t2;
            m_Labels[1] = l2;
            m_Texts[2] = t3;
            m_Labels[2] = l3;
            m_Texts[3] = t4;
            m_Labels[3] = l4;

            // MSChart の設定
            chart1.Series.Clear();
            chart1.ChartAreas.Clear();
            ChartArea chartArea1 = new ChartArea();
            chart1.ChartAreas.Add(chartArea1);

            // 乱数発生器
            cmbGen.Items.Add(new Gen(RandomNumber.GENERATOR.MCG31));
            cmbGen.Items.Add(new Gen(RandomNumber.GENERATOR.MCG59));
            cmbGen.Items.Add(new Gen(RandomNumber.GENERATOR.MRG32K3A));
            cmbGen.Items.Add(new Gen(RandomNumber.GENERATOR.MT19937));
            cmbGen.Items.Add(new Gen(RandomNumber.GENERATOR.MT2203));
            cmbGen.Items.Add(new Gen(RandomNumber.GENERATOR.NIEDERR));
            cmbGen.Items.Add(new Gen(RandomNumber.GENERATOR.R250));
            cmbGen.Items.Add(new Gen(RandomNumber.GENERATOR.SFMT19937));
            cmbGen.Items.Add(new Gen(RandomNumber.GENERATOR.SOBOL));
            cmbGen.Items.Add(new Gen(RandomNumber.GENERATOR.WH));
            cmbGen.SelectedIndex = 0;

            // 分布関数
            cmbDist.Items.Add(new Dist(R_METHOD.Uniform, new string[] { "連続：Uniform", "区間の開始点", "区間の終了点" }));
            cmbDist.Items.Add(new Dist(R_METHOD.Gaussian, new string[] { "連続：Gausian", "平均値", "標準偏差 >0 " }));
            cmbDist.Items.Add(new Dist(R_METHOD.Exponential, new string[] { "連続：Exponential", "位置因子", "尺度因子 >0 " }));
            cmbDist.Items.Add(new Dist(R_METHOD.Laplace, new string[] { "連続：Laplace", "平均値", "尺度因子 >0 " }));
            cmbDist.Items.Add(new Dist(R_METHOD.Weibull, new string[] { "連続：Weibull", "位置因子", "形状因子 >0 ", "尺度因子 >0 " }));
            cmbDist.Items.Add(new Dist(R_METHOD.Cauchy, new string[] { "連続：Cauchy", "位置因子", "尺度因子 >0 " }));
            cmbDist.Items.Add(new Dist(R_METHOD.Rayleigh, new string[] { "連続：Rayleigh", "位置因子", "尺度因子 >0 " }));
            cmbDist.Items.Add(new Dist(R_METHOD.Lognormal, new string[] { "連続：Lognormal", "平均", "標準偏差 >0 ", "位置因子", "尺度因子 >0 " }));
            cmbDist.Items.Add(new Dist(R_METHOD.Gumbel, new string[] { "連続：Gumbel", "位置因子", "尺度因子 >0 " }));
            cmbDist.Items.Add(new Dist(R_METHOD.Gamma, new string[] { "連続：Gamma", "形状因子 >0 ", "位置因子", "尺度因子 >0 " }));
            cmbDist.Items.Add(new Dist(R_METHOD.Beta, new string[] { "連続：Beta", "形状因子1 >0 ", "形状因子2 >0 ", "位置因子", "尺度因子 >0 " }));

            cmbDist.Items.Add(new Dist(R_METHOD.UniformI, new string[] { "離散：Uniform", "開始点(整数)", "終了点(整数)"}));
            cmbDist.Items.Add(new Dist(R_METHOD.UniformBits, new string[] { "離散：UniformBits"}));
            cmbDist.Items.Add(new Dist(R_METHOD.UniformBits32, new string[] { "離散：UniformBits32" }));
            cmbDist.Items.Add(new Dist(R_METHOD.UniformBits64, new string[] { "離散：UniformBits64" }));
            cmbDist.Items.Add(new Dist(R_METHOD.Bernoulli, new string[] { "離散：Bernoulli", "成功確率" }));
            cmbDist.Items.Add(new Dist(R_METHOD.Geometric, new string[] { "離散：Geometric", "成功確率" }));
            cmbDist.Items.Add(new Dist(R_METHOD.Binomial, new string[] { "離散：Binomial", "試行回数 > 0", "成功確率" }));
            cmbDist.Items.Add(new Dist(R_METHOD.Hypergeometric, new string[] { "離散：Hypergeometric", "母集団の大きさ", "抽出個数", "「あたり」個数" }));
            cmbDist.Items.Add(new Dist(R_METHOD.Poisson, new string[] { "離散：Poisson", "分布パラメータ" }));
            cmbDist.Items.Add(new Dist(R_METHOD.NegBinomial, new string[] { "離散：NegBinomial", "位置因子１ > 0", "0 < 位置因子２ < 1" }));
            cmbDist.SelectedIndex = 0;
            btnDraw_Click(null, null);
        }

        private void btnDraw_Click(object sender, EventArgs e)
        {
            // Gausian f(x) = 1/(sqrt(2π)*std_dev) * exp(-(x-mean)**2/(2std_dev**2))
            int N = 2000; // データレコード数
            int M = 100;  // プロット点数
            Gen g = (Gen)cmbGen.SelectedItem;
            uint seed = uint.Parse(txtSeed.Text);
            RandomNumber rnd = new RandomNumber(g.gen, seed);
            VectorDenseDouble vals = new VectorDenseDouble(N);
            try
            {
                // 指定乱数を発生させる
                Dist d = Generate(rnd, vals);

                // 乱数の統計量の計算
                MatrixDenseGeneralDouble data = new MatrixDenseGeneralDouble(N, 1);
                data.Columns[1] = vals;
                Stat stat = new Stat(data);
                stat.Computed = Stat.PROPERTIES.MIN | Stat.PROPERTIES.MAX | Stat.PROPERTIES.VARIANCE | Stat.PROPERTIES.MOMENT2;
                stat.ComputeProperties();

                double min = Math.Floor(stat.Min[1]);
                double max = Math.Ceiling(stat.Max[1]);

                int[] freq = new int[M];
                int mf = Freq(freq, min, max, vals);
                txtResult.Text = "最小：　　  " + stat.Min[1] + "\r\n最大：　　  " + stat.Max[1] + "\r\n最頻：　　  " + mf +
                                 "\r\n平均：　　  " + stat.Mean[1].ToString("G5") +
                                 "\r\n分散：　　  " + stat.Variance[1].ToString("G5") +
                                 "\r\n標準偏差：" + Math.Sqrt(stat.Variance[1]).ToString("G5");
                Draw(d, stat, freq, min, max);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }

        private Dist Generate(RandomNumber rnd, VectorDenseDouble vals)
        {
            Dist d = (Dist)cmbDist.SelectedItem;
            double v1 = double.Parse(t1.Text);
            double v2 = double.Parse(t2.Text);
            double v3 = double.Parse(t3.Text);
            double v4 = double.Parse(t4.Text);

            _Array.Fill(vals, new System.Numerics.Complex(0.0, 0.0));
            VectorDenseDouble val1 = new VectorDenseDouble(vals.SizeLimit);

            for (int i = 0; i < m_sampling_size; ++i)
            {
                switch (d.Method)
                {
                    //  連続分布
                    case R_METHOD.Uniform:
                        rnd.GenerateUniform(val1, v1, v2);
                        break;
                    case R_METHOD.Gaussian:
                        rnd.GenerateGaussian(val1, v1, v2, RandomNumber.GAUSSIAN_METHOD.BOXMULLER);
                        break;
                    case R_METHOD.Exponential:
                        rnd.GenerateExponential(val1, v1, v2);
                        break;
                    case R_METHOD.Laplace:
                        rnd.GenerateLaplace(val1, v1, v2);
                        break;
                    case R_METHOD.Weibull:
                        rnd.GenerateWeibull(val1, v1, v2, v3);
                        break;
                    case R_METHOD.Cauchy:
                        rnd.GenerateCauchy(val1, v1, v2);
                        break;
                    case R_METHOD.Rayleigh:
                        rnd.GenerateRayleigh(val1, v1, v2);
                        break;
                    case R_METHOD.Lognormal:
                        rnd.GenerateLognormal(val1, v1, v2, v3, v4, RandomNumber.LOGNORMAL_METHOD.ICDF);
                        break;
                    case R_METHOD.Gumbel:
                        rnd.GenerateGumbel(val1, v1, v2);
                        break;
                    case R_METHOD.Gamma:
                        rnd.GenerateGamma(val1, v1, v2, v3);
                        break;
                    case R_METHOD.Beta:
                        rnd.GenerateBeta(val1, v1, v2, v3, v4);
                        break;

                    //　離散分布
                    case R_METHOD.UniformI:     // 整数区間
                        rnd.GenerateUniform(val1, (int)v1, (int)v2);
                        break;
                    case R_METHOD.UniformBits:
                        rnd.GenerateUniformBits(val1);
                        break;
                    case R_METHOD.UniformBits32:
                        rnd.GenerateUniformBits32(val1);
                        break;
                    case R_METHOD.UniformBits64:
                        rnd.GenerateUniformBits64(val1);
                        break;
                    case R_METHOD.Bernoulli:
                        rnd.GenerateBernoulli(val1, v1);
                        break;
                    case R_METHOD.Geometric:
                        rnd.GenerateGeometric(val1, v1);
                        break;
                    case R_METHOD.Binomial:
                        rnd.GenerateBinomial(val1, (int)v1, v2);
                        break;
                    case R_METHOD.Hypergeometric:
                        rnd.GenerateHypergeometric(val1, (int)v1, (int)v2, (int)v3);
                        break;
                    case R_METHOD.Poisson:
                        rnd.GeneratePoisson(val1, v1, RandomNumber.POISSON_METHOD.POISNORM);
                        break;
                    /*
                case R_METHOD.PoissonV:
                    break;
                     */
                    case R_METHOD.NegBinomial:
                        rnd.GenerateNegbinomial(val1, v1, v2);
                        break;
                }
                _VML.Add(vals, val1);
            }
            _Array.Div(vals, vals, (double)m_sampling_size); // 平均
            return d;
        }

        private void Draw(Dist d,Stat stat,int [] freq,double min, double max)
        {
            chart1.Series.Clear();
            Series series = new Series(d.Titles[0]);
            Series seriesN = new Series("正規分布");
            double A = 1.0 / Math.Sqrt(2.0 * 3.141592653589793238462*stat.Variance[1]);
            int M = freq.Length;
            double dx = (max-min)/M;

            // 頻度から確率密度に変換
            double t = 0;
            for (int i = 0; i < M; ++i)
            {
                t += freq[i];
            }

            double ym = 0;
            for (int i = 0; i < M; ++i)
            {
                double y = freq[i] / t / dx; // 矩形 y*dx が（矩形の）確率になるように y を全面積で割って確率密度に変換する。、
                if (ym < y) ym = y;
                double x = min + dx * i+dx/2;
                double dm = x - stat.Mean[1];
                DataPoint dp = new DataPoint(x, y);
                DataPoint dpN = new DataPoint(x,A*Math.Exp(-dm*dm/(2.0*stat.Variance[1])));
                dp.MarkerStyle = MarkerStyle.Diamond;
                dpN.MarkerStyle = MarkerStyle.Circle;
                series.Points.Add(dp);
                seriesN.Points.Add(dpN);
            }

            series.ChartType = SeriesChartType.Point | SeriesChartType.Line;
            seriesN.ChartType = SeriesChartType.Point | SeriesChartType.Line;
            chart1.ChartAreas[0].AxisX.Minimum = min;
            chart1.ChartAreas[0].AxisX.Maximum = max;
            chart1.ChartAreas[0].AxisY.Minimum = 0;
            chart1.ChartAreas[0].AxisY.Maximum = ym*1.05;
            chart1.Series.Add(series);
            chart1.Series.Add(seriesN);
            // 折れ線グラフの描画
            chart1.Update();
        }

        // 区間頻度の計算 (戻り値は最大の頻度数)
        private int Freq(int [] freq,double min,double max,VectorDenseDouble vals)
        {
            int n = vals.Count;
            int m = freq.Length-1;
            double dx = (max - min) / m;
            int fmax = 0;

            if (dx <= 0)
            {
                for (int i = 0; i < m; ++i) freq[0] = 1;
                return freq.Length;
            }
            for (int i = 0; i < m; ++i) freq[0] = 0;

            for (int i = 1; i <= n; ++i)
            {
                int j = (int)((vals[i] - min)/dx);
                freq[j]++;
                if (fmax < freq[j]) fmax = freq[j];
            }
            return fmax;
        }

        private void cmbDist_SelectedIndexChanged(object sender, EventArgs e)
        {
            Dist dist = (Dist)cmbDist.SelectedItem;
            string[] titles = dist.Titles;
            for (int i = 1; i <= 4; ++i)
            {
                if(i<titles.Length)
                {
                    m_Texts[i - 1].Visible = true;
                    m_Texts[i - 1].Enabled = true;
                    m_Labels[i - 1].Visible = true;
                    m_Labels[i - 1].Text = titles[i];
                } else {
                    m_Texts[i - 1].Visible = false;
                    m_Labels[i - 1].Visible = false;
                }
            }
        }

        private void txtSeed_Validating(object sender, CancelEventArgs e)
        {
            uint v;
            if (uint.TryParse(txtSeed.Text, out v)) return;
            e.Cancel = true;
        }

        private void t1_Validating(object sender, CancelEventArgs e)
        {
            double v;
            if (double.TryParse(t1.Text, out v)) return;
            e.Cancel = true;
        }

        private void t2_Validating(object sender, CancelEventArgs e)
        {
            double v;
            if (double.TryParse(t2.Text, out v)) return;
            e.Cancel = true;
        }

        private void t3_Validating(object sender, CancelEventArgs e)
        {
            double v;
            if (double.TryParse(t3.Text, out v)) return;
            e.Cancel = true;
        }

        private void t4_Validating(object sender, CancelEventArgs e)
        {
            double v;
            if (double.TryParse(t4.Text, out v)) return;
            e.Cancel = true;
        }

        private void txtN_Validating(object sender, CancelEventArgs e)
        {
            int v;
            if (int.TryParse(txtN.Text, out v))
            {
                if (v > 0)
                {
                    m_sampling_size = v;
                    return;
                }
            }
            e.Cancel = true;
        }
    }
}
