﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Diagnostics;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Windows.Forms.DataVisualization.Charting;

using Mol;

namespace Examples
{
    public partial class FormChart : Form
    {
        private Chart                    m_Chart;  // MSChart コントロール
        private static int               m_nx = 5; // データ点の数
        private static int               m_ny = 3; // データ点 x １個に対する関数値 y の数 
        private static int               m_np = m_nx*10 + 10 + 1; // 計算点の数（含む、データ点範囲の前後１０個）
        private VectorDenseDouble        m_x  = new VectorDenseDouble(m_nx);
        private MatrixDenseGeneralDouble m_ys = new MatrixDenseGeneralDouble(m_nx,m_ny);
        private VectorDenseDouble        m_xp = new VectorDenseDouble(m_np); // 計算点
        // 全計算点における全関数値の行列。
        private MatrixDenseGeneralDouble m_res = new MatrixDenseGeneralDouble(m_np,m_ny);
        private double m_dx = 0.1; // 計算点の幅（(m_nx+2)*10+1個でデータ点の全幅を含む）
        private double m_x0 = 0.0; // 先頭の計算点

        private DataFitting m_df1; // 関数値にベクトル
        private DataFitting m_df2; // 関数値に行列

        public FormChart()
        {
            InitializeComponent();
        }

        /// <summary>
        /// m_df1とm_df2に設定されたデータ点列を元に補間計算を実行して折れ線グラフを描く。
        /// </summary>
        /// <param name="title">描画タイトル（スプラインのタイプ）</param>
        private void Draw(string title)
        {
            Series[] serieses = new Series[m_ny];
            for (int i = 1; i <= m_ny; ++i)
            {
                serieses[i-1] = new Series();
                serieses[i-1].Points.Clear();
                serieses[i-1].Name = title+" - Y"+i;
            }

            // 補間の実行
            MatrixDenseGeneralDouble y = (MatrixDenseGeneralDouble)m_df2.ComputeAt(m_xp);
 
            for (int i = 1; i <= m_np; ++i)
            {
                double xp = m_xp[i];
                bool fMarker = false;
                for (int k = 1; k <= m_x.Count; ++k)
                {
                    // 実際のデータ点にマーカーを付ける
                    if (Math.Abs(xp - m_x[k]) < 1.0e-10)
                    {
                        fMarker = true;
                        break;
                    }
                }
                // 単純な確認
                double y1 = (double)m_df1.ComputeAt(xp);
                T.EQ(y1, y[i, 2]);
                VectorDenseDouble ys = (VectorDenseDouble)m_df2.ComputeAt(xp);

                for (int j = 1; j <= m_ny; ++j)
                {
                    T.EQ(ys[j], y[i, j]); // 確認
                    DataPoint dp = new DataPoint(xp, y[i, j]);
                    if(fMarker) dp.MarkerStyle = MarkerStyle.Circle;
                    serieses[j - 1].Points.Add(dp);
                }
            }

            // 描画折れ線グラフの選択
            if (chk1.Checked)
            {
                serieses[0].ChartType = SeriesChartType.Line;
                m_Chart.Series.Add(serieses[0]);
            }
            if (chk2.Checked)
            {
                serieses[1].ChartType = SeriesChartType.Line;
                m_Chart.Series.Add(serieses[1]);
            }
            if (chk3.Checked)
            {
                serieses[2].ChartType = SeriesChartType.Line;
                m_Chart.Series.Add(serieses[2]);
            }
            // 折れ線グラフの描画
            m_Chart.Update();

            // 積分値の確認
            VectorDenseDouble iv = new VectorDenseDouble(m_ny); // 近似値の計算用
            VectorDenseDouble ivc = (VectorDenseDouble)m_df2.Integrate(m_x0, m_x0 + m_dx * (m_np-1)); // 積分値の計算
            // 近似値の計算
            for (int j = 1; j <= m_ny; ++j)
            {
                iv[j]  = (y[1   , j] / 2) * m_dx;
                iv[j] += (y[m_np, j] / 2) * m_dx;
            }
            for (int i = 2; i < m_np; ++i)
            {
                for (int j = 1; j <= m_ny;++j)
                {
                    iv[j] += y[i, j] * m_dx;
                }
            }
            // 計算値 - 近似値
            txtInt1.Text = "" + (ivc[1] - iv[1]);
            txtInt2.Text = "" + (ivc[2] - iv[2]);
            txtInt3.Text = "" + (ivc[3] - iv[3]);
        }

        private void FormChart_Load(object sender, EventArgs e)
        {
            chk1.Checked = true;

            m_Chart = chart1;

            // データ点と対応する関数値の初期セット
            m_x[1] = 1; m_ys[1,2] =  0;
            m_x[2] = 2; m_ys[2,2] =  2;
            m_x[3] = 3; m_ys[3,2] = -1;
            m_x[4] = 4; m_ys[4,2] =  -0.5;
            m_x[5] = 5; m_ys[5,2] =  0;    // 周期スプライン用に始点と終点の関数値を合わせる。
            m_ys.Columns[1] = (VectorDenseDouble)_Array.Add(null, m_ys.Columns[2], 1.0);
            m_ys.Columns[3] = (VectorDenseDouble)_Array.Sub(null, m_ys.Columns[2], 1.0);

            dataGridView1.ColumnCount = m_ys.ColumnCount;
            dataGridView1.RowCount    = m_ys.RowCount;
            for(int i=0;i<dataGridView1.RowCount;++i)
            {
                for(int j=0;j<dataGridView1.ColumnCount;++j)
                {
                    dataGridView1.Rows[i].Cells[j].Value = "" + m_ys[i+1, j+1];
                }
                dataGridView1.Rows[i].HeaderCell.Value = ""+ m_x[i + 1];
            }
            for (int i = 0; i < m_ys.ColumnCount; ++i)
            {
                dataGridView1.Columns[i].HeaderText = "Y" + (i+1)+"(X)";
            }
            dataGridView1.TopLeftHeaderCell.Value = "X";
            dataGridView1.ReadOnly = false;

            // 計算点のセット
            m_dx = 0.1;
            m_x0 = 0;
            for(int i=1;i<=m_np;++i)
            {
                m_xp[i] = m_x0 + (i - 1) * m_dx; 
            }

            // MSChart の設定
            m_Chart.Series.Clear();
            m_Chart.ChartAreas.Clear();
            ChartArea chartArea1 = new ChartArea();
            chartArea1.AxisX.Minimum = m_x[1]-1.0;
            chartArea1.AxisX.Maximum = m_x[m_x.Count]+1.0;
            chartArea1.AxisX.MajorTickMark.Interval = 1.0;
            chartArea1.AxisX.Interval = 1.0;
            chartArea1.AxisY.Minimum = -5.0;
            chartArea1.AxisY.Maximum = 5.0;
            chartArea1.AxisY.Interval = 1.0;
            m_Chart.ChartAreas.Add(chartArea1);
        }

        private void btnDraw_Click(object sender, EventArgs e)
        {
            string title = "";
            m_Chart.Series.Clear();
            try
            {
                // Y データの取出し
                for (int i = 0; i < dataGridView1.RowCount; ++i)
                {
                    for (int j = 0; j < dataGridView1.ColumnCount; ++j)
                    {
                        m_ys[i + 1, j + 1] = double.Parse(dataGridView1.Rows[i].Cells[j].Value.ToString());
                    }
                }

                // DataFitting オブジェクトの作成
                // 一つのX に Y は一個、、従って ComputeAt() に double の x なら double、ベクトルの x ならベクトルの y が返る。
                m_df1 = new DataFitting(m_x, m_ys.Columns[2]); 
                // 一つのX に Y は３個、従って ComputeAt() に double の x ならベクトル、ベクトルの x なら行列の y が返る。
                m_df2 = new DataFitting(m_x, m_ys);            

                // 多項式（スプライン）等のセット
                if (rdoStepWise.Checked)
                {
                    title = "階段関数";
                    SplineStepWise sp = new SplineStepWise();
                    if (rdoL.Checked) sp.IncludedEdge = SplineStepWise.INCLUDED_EDGE.LEFT;
                    if (rdoR.Checked) sp.IncludedEdge = SplineStepWise.INCLUDED_EDGE.RIGHT;
                    // スプライン関数のセット
                    m_df1.SetSpline(sp);
                    m_df2.SetSpline(sp);
                }
                if (rdoLinear.Checked)
                {
                    title = "線形関数";
                    SplineLinear sp = new SplineLinear();
                    // スプライン関数のセット
                    m_df1.SetSpline(sp);
                    m_df2.SetSpline(sp);
                }
                if (rdoQuadratic.Checked)
                {
                    title = "２次関数";
                    SplineQuadratic sq = new SplineQuadratic();
                    sq.BoundaryValue = double.Parse(txt2V.Text.Trim());
                    // スプライン関数のセット
                    m_df1.SetSpline(sq);
                    m_df2.SetSpline(sq);
                }
                if (rdoCubic.Checked)
                {
                    title = "３次関数";
                    SplineCubic sc = new SplineCubic();
                    sc.LeftBoundaryValue = double.Parse(txtValL.Text.Trim());
                    sc.RightBoundaryValue = double.Parse(txtValR.Text.Trim());

                    if (rdoSingle.Checked)
                    {
                        // 単一境界条件
                        if (rdoNOT.Checked     ) sc.SetBoundaryCondition(SplineCubic.SINGLE_BOUNDARY_CONDITION.NOT_A_KNOT);
                        if (rdoPeriodic.Checked) sc.SetBoundaryCondition(SplineCubic.SINGLE_BOUNDARY_CONDITION.PERIODIC);
                        if (rdoFreeEnd.Checked ) sc.SetBoundaryCondition(SplineCubic.SINGLE_BOUNDARY_CONDITION.FREE_END);
                    }
                    else
                    {
                        // 両端点条件
                        SplineCubic.EDGE_BOUNDARY_CONDITION el = SplineCubic.EDGE_BOUNDARY_CONDITION.SECOND_DERIVATIVE;
                        SplineCubic.EDGE_BOUNDARY_CONDITION er = SplineCubic.EDGE_BOUNDARY_CONDITION.SECOND_DERIVATIVE;
                        double bl = double.Parse(txtValL.Text.Trim());
                        double br = double.Parse(txtValR.Text.Trim());
                        if (rdo1LD.Checked) el = SplineCubic.EDGE_BOUNDARY_CONDITION.FIRST_DERIVATIVE;
                        if (rdo2LD.Checked) el = SplineCubic.EDGE_BOUNDARY_CONDITION.SECOND_DERIVATIVE;
                        if (rdo1RD.Checked) er = SplineCubic.EDGE_BOUNDARY_CONDITION.FIRST_DERIVATIVE;
                        if (rdo2RD.Checked) er = SplineCubic.EDGE_BOUNDARY_CONDITION.SECOND_DERIVATIVE;
                        sc.SetBoundaryCondition(el, bl, er, br);
                    }
                    // スプライン関数のセット
                    m_df1.SetSpline(sc);
                    m_df2.SetSpline(sc);
                }

                // 描画
                Draw(title);

            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }

        }
    }
}
