﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Diagnostics;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Windows.Forms.DataVisualization.Charting;

using Mol;

namespace Examples
{
    public partial class FormRand : Form
    {
        public FormRand()
        {
            InitializeComponent();
        }
        public TextBox[] m_Texts  = new TextBox[4];
        public Label  [] m_Labels = new Label[4];

        public enum R_METHOD
        {
            // 連続分布
            Uniform = 1,
            Gaussian,
            Exponential,
            Laplace,
            Weibull,
            Cauchy,
            Rayleigh,
            Lognormal,
            Gumbel,
            Gamma,
            Beta,
            // 離散分布
            UniformI,     // 整数区間
            UniformBits,
            UniformBits32,
            UniformBits64,
            Bernoulli,
            Geometric,
            Binomial,
            Hypergeometric,
            Poisson,
            PoissonV,
            NegBinomial
        }

        /// <summary>
        /// 疑似乱数発生器のコンボボックス用クラス。
        /// </summary>
        public class Gen
        {
            public RandomNumber.GENERATOR gen;
            public Gen(RandomNumber.GENERATOR g)
            {
                gen = g;
            }
            public override string ToString()
            {
                return gen.ToString();
            }
        }

        /// <summary>
        /// 分布関数選択コンボボックス用クラス。
        /// </summary>
        public class Dist
        {
            public string[] Titles;
            public R_METHOD Method;
            public Dist(R_METHOD method,string[] titles)
            {
                Method = method;
                Titles = titles;
            }
            public override string ToString()
            {
                return Titles[0];
            }
        }
        private void FormRand_Load(object sender, EventArgs e)
        {
            m_Texts[0] = t1;
            m_Labels[0] = l1;
            m_Texts[1] = t2;
            m_Labels[1] = l2;
            m_Texts[2] = t3;
            m_Labels[2] = l3;
            m_Texts[3] = t4;
            m_Labels[3] = l4;

            // MSChart の設定
            chart1.Series.Clear();
            chart1.ChartAreas.Clear();
            ChartArea chartArea1 = new ChartArea();
            chart1.ChartAreas.Add(chartArea1);
            // Generator
            cmbGen.Items.Add(new Gen(RandomNumber.GENERATOR.MCG31));
            cmbGen.Items.Add(new Gen(RandomNumber.GENERATOR.MCG59));
            cmbGen.Items.Add(new Gen(RandomNumber.GENERATOR.MRG32K3A));
            cmbGen.Items.Add(new Gen(RandomNumber.GENERATOR.MT19937));
            cmbGen.Items.Add(new Gen(RandomNumber.GENERATOR.MT2203));
            cmbGen.Items.Add(new Gen(RandomNumber.GENERATOR.NIEDERR));
            cmbGen.Items.Add(new Gen(RandomNumber.GENERATOR.R250));
            cmbGen.Items.Add(new Gen(RandomNumber.GENERATOR.SFMT19937));
            cmbGen.Items.Add(new Gen(RandomNumber.GENERATOR.SOBOL));
            cmbGen.Items.Add(new Gen(RandomNumber.GENERATOR.WH));
            cmbGen.SelectedIndex = 0;
            // 分布
            cmbDist.Items.Add(new Dist(R_METHOD.Uniform, new string[] { "連続：Uniform", "区間の開始点", "区間の終了点" }));
            cmbDist.Items.Add(new Dist(R_METHOD.Gaussian, new string[] { "連続：Gausian", "平均値", "標準偏差 >0 " }));
            cmbDist.Items.Add(new Dist(R_METHOD.Exponential, new string[] { "連続：Exponential", "位置因子", "尺度因子 >0 " }));
            cmbDist.Items.Add(new Dist(R_METHOD.Laplace, new string[] { "連続：Laplace", "平均値", "尺度因子 >0 " }));
            cmbDist.Items.Add(new Dist(R_METHOD.Weibull, new string[] { "連続：Weibull", "位置因子", "形状因子 >0 ", "尺度因子 >0 " }));
            cmbDist.Items.Add(new Dist(R_METHOD.Cauchy, new string[] { "連続：Cauchy", "位置因子", "尺度因子 >0 " }));
            cmbDist.Items.Add(new Dist(R_METHOD.Rayleigh, new string[] { "連続：Rayleigh", "位置因子", "尺度因子 >0 " }));
            cmbDist.Items.Add(new Dist(R_METHOD.Lognormal, new string[] { "連続：Lognormal", "平均", "標準偏差 >0 ", "位置因子", "尺度因子 >0 " }));
            cmbDist.Items.Add(new Dist(R_METHOD.Gumbel, new string[] { "連続：Gumbel", "位置因子", "尺度因子 >0 " }));
            cmbDist.Items.Add(new Dist(R_METHOD.Gamma, new string[] { "連続：Gamma", "形状因子 >0 ", "位置因子", "尺度因子 >0 " }));
            cmbDist.Items.Add(new Dist(R_METHOD.Beta, new string[] { "連続：Beta", "形状因子1 >0 ", "形状因子2 >0 ", "位置因子", "尺度因子 >0 " }));

            cmbDist.Items.Add(new Dist(R_METHOD.UniformI, new string[] { "離散：Uniform", "開始点(整数)", "終了点(整数)"}));
            cmbDist.Items.Add(new Dist(R_METHOD.UniformBits, new string[] { "離散：UniformBits"}));
            cmbDist.Items.Add(new Dist(R_METHOD.UniformBits32, new string[] { "離散：UniformBits32" }));
            cmbDist.Items.Add(new Dist(R_METHOD.UniformBits64, new string[] { "離散：UniformBits64" }));
            cmbDist.Items.Add(new Dist(R_METHOD.Bernoulli, new string[] { "離散：Bernoulli", "成功確率" }));
            cmbDist.Items.Add(new Dist(R_METHOD.Geometric, new string[] { "離散：Geometric", "成功確率" }));
            cmbDist.Items.Add(new Dist(R_METHOD.Binomial, new string[] { "離散：Binomial", "試行回数 > 0", "成功確率" }));
            cmbDist.Items.Add(new Dist(R_METHOD.Hypergeometric, new string[] { "離散：Hypergeometric", "母集団の大きさ", "抽出個数", "「あたり」個数" }));
            cmbDist.Items.Add(new Dist(R_METHOD.Poisson, new string[] { "離散：Poisson", "分布パラメータ" }));
            cmbDist.Items.Add(new Dist(R_METHOD.NegBinomial, new string[] { "離散：NegBinomial", "位置因子１ > 0", "0 < 位置因子２ < 1" }));
            cmbDist.SelectedIndex = 0;
            btnDraw_Click(null, null);
        }

        private void btnDraw_Click(object sender, EventArgs e)
        {
            // Gausian f(x) = 1/(sqrt(2π)*std_dev) * exp(-(x-mean)**2/(2std_dev**2))
            int N = 2000; // データレコード数
            int M = 100; // プロット点数
            Gen g = (Gen)cmbGen.SelectedItem;
            uint seed = uint.Parse(txtSeed.Text);
            RandomNumber rnd = new RandomNumber(g.gen, seed);
            VectorDenseDouble vals = new VectorDenseDouble(N);
            try
            {
                //　コンボボックスで指定された条件での乱数発生
                Dist d = Generate(rnd, vals);

                //　発生した乱数の統計量の計算
                MatrixDenseGeneralDouble data = new MatrixDenseGeneralDouble(N, 1);
                data.Columns[1] = vals;
                Stat stat = new Stat(data);
                stat.Computed = Stat.PROPERTIES.MIN | Stat.PROPERTIES.MAX | Stat.PROPERTIES.VARIANCE | Stat.PROPERTIES.MOMENT2;
                stat.ComputeProperties();

                // 最小～最大値までの区間に落ちる頻度を Y-軸 にして頻度グラフの描画
                double min = Math.Floor(stat.Min[1]);
                double max = Math.Ceiling(stat.Max[1]);

                int[] freq = new int[M];
                chart1.ChartAreas[0].AxisY.Minimum = 0;
                int mf = Freq(freq, min, max, vals); 
                txtResult.Text = "最小：　　  " + stat.Min[1] + "\r\n最大：　　  " + stat.Max[1] + "\r\n最頻：　　  " + mf+

                                 "\r\n平均：　　  " + stat.Mean[1] + "\r\n分散：　　  " + stat.Variance[1] +
                                 "\r\n標準偏差：" + Math.Sqrt(stat.Variance[1]);
                   mf = (mf + 9) / 10;
                chart1.ChartAreas[0].AxisY.Maximum = mf * 10;
                Draw(d.Titles[0], freq, min, max);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }

        private Dist Generate(RandomNumber rnd, VectorDenseDouble vals)
        {
            Dist d = (Dist)cmbDist.SelectedItem;
            double v1 = double.Parse(t1.Text);
            double v2 = double.Parse(t2.Text);
            double v3 = double.Parse(t3.Text);
            double v4 = double.Parse(t4.Text);

            switch (d.Method)
            {
                    //  連続分布
                case R_METHOD.Uniform:
                    rnd.GenerateUniform(vals, v1, v2);
                    break;
                case R_METHOD.Gaussian:
                    rnd.GenerateGaussian(vals, v1, v2, RandomNumber.GAUSSIAN_METHOD.BOXMULLER);
                    break;
                case R_METHOD.Exponential:
                    rnd.GenerateExponential(vals, v1, v2);
                    break;
                case R_METHOD.Laplace:
                    rnd.GenerateLaplace(vals, v1, v2);
                    break;
                case R_METHOD.Weibull:
                    rnd.GenerateWeibull(vals, v1, v2,v3);
                    break;
                case R_METHOD.Cauchy:
                    rnd.GenerateCauchy(vals, v1, v2);
                    break;
                case R_METHOD.Rayleigh:
                    rnd.GenerateRayleigh(vals, v1, v2);
                    break;
                case R_METHOD.Lognormal:
                    rnd.GenerateLognormal(vals, v1, v2, v3,v4,RandomNumber.LOGNORMAL_METHOD.ICDF);
                    break;
                case R_METHOD.Gumbel:
                    rnd.GenerateGumbel(vals, v1, v2);
                    break;
                case R_METHOD.Gamma:
                    rnd.GenerateGamma(vals, v1, v2, v3);
                    break;
                case R_METHOD.Beta:
                    rnd.GenerateBeta(vals, v1, v2, v3, v4);
                    break;

                    //　離散分布
                case R_METHOD.UniformI:     // 整数区間
                    rnd.GenerateUniform(vals, (int)v1, (int)v2);
                    break;
                case R_METHOD.UniformBits:
                    rnd.GenerateUniformBits(vals);
                    break;
                case R_METHOD.UniformBits32:
                    rnd.GenerateUniformBits32(vals);
                    break;
                case R_METHOD.UniformBits64:
                    rnd.GenerateUniformBits64(vals);
                    break;
                case R_METHOD.Bernoulli:
                    rnd.GenerateBernoulli(vals,v1);
                    break;
                case R_METHOD.Geometric:
                    rnd.GenerateGeometric(vals, v1);
                    break;
                case R_METHOD.Binomial:
                    rnd.GenerateBinomial(vals,(int)v1,v2);
                    break;
                case R_METHOD.Hypergeometric:
                    rnd.GenerateHypergeometric(vals, (int)v1, (int)v2, (int)v3);
                    break;
                case R_METHOD.Poisson:
                    rnd.GeneratePoisson(vals, v1, RandomNumber.POISSON_METHOD.POISNORM);
                    break;
                    /*
                case R_METHOD.PoissonV:
                    break;
                     */
                case R_METHOD.NegBinomial:
                    rnd.GenerateNegbinomial(vals, v1, v2);
                    break;
            }
            return d;
        }

        private void Draw(string title,int [] freq,double min, double max)
        {
            chart1.Series.Clear();
            Series series = new Series(title);
            int M = freq.Length;
            double dx = (max-min)/M;

            for (int i = 0; i <M; ++i)
            {
                DataPoint dp = new DataPoint(min + dx * i+dx/2, freq[i]);
                dp.MarkerStyle = MarkerStyle.Circle;
                series.Points.Add(dp);
            }
            series.ChartType = SeriesChartType.Point | SeriesChartType.Line;
            chart1.ChartAreas[0].AxisX.Minimum = min;
            chart1.ChartAreas[0].AxisX.Maximum = max;

            chart1.Series.Add(series);
            // 折れ線グラフの描画
            chart1.Update();
        }

        // 区間頻度の計算
        private int Freq(int [] freq,double min,double max,VectorDenseDouble vals)
        {
            int n = vals.Count;
            int m = freq.Length-1;
            double dx = (max - min) / m;
            int fmax = 0;

            if (dx <= 0)
            {
                for (int i = 0; i < m; ++i) freq[0] = 1;
                return freq.Length;
            }
            for (int i = 0; i < m; ++i) freq[0] = 0;

            for (int i = 1; i <= n; ++i)
            {
                int j = (int)((vals[i] - min)/dx);
                freq[j]++;
                if (fmax < freq[j]) fmax = freq[j];
            }
            return fmax;
        }

        private void cmbDist_SelectedIndexChanged(object sender, EventArgs e)
        {
            Dist dist = (Dist)cmbDist.SelectedItem;
            string[] titles = dist.Titles;
            for (int i = 1; i <= 4; ++i)
            {
                if(i<titles.Length)
                {
                    m_Texts[i - 1].Visible = true;
                    m_Texts[i - 1].Enabled = true;
                    m_Labels[i - 1].Visible = true;
                    m_Labels[i - 1].Text = titles[i];
                } else {
                    m_Texts[i - 1].Visible = false;
                    m_Labels[i - 1].Visible = false;
                }
            }
        }

        private void txtSeed_Validating(object sender, CancelEventArgs e)
        {
            uint v;
            if (uint.TryParse(txtSeed.Text, out v)) return;
            e.Cancel = true;
        }

        private void t1_Validating(object sender, CancelEventArgs e)
        {
            double v;
            if (double.TryParse(t1.Text, out v)) return;
            e.Cancel = true;
        }

        private void t2_Validating(object sender, CancelEventArgs e)
        {
            double v;
            if (double.TryParse(t2.Text, out v)) return;
            e.Cancel = true;
        }

        private void t3_Validating(object sender, CancelEventArgs e)
        {
            double v;
            if (double.TryParse(t3.Text, out v)) return;
            e.Cancel = true;
        }

        private void t4_Validating(object sender, CancelEventArgs e)
        {
            double v;
            if (double.TryParse(t4.Text, out v)) return;
            e.Cancel = true;
        }
    }
}
