﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Diagnostics;
using System.Drawing;
using System.Linq;
using System.Numerics;
using System.Text;
using System.Windows.Forms;

using Mol;

namespace Examples
{
    /// <summary>
    /// 情報のコンソール出力（Consoleが有効であること）や Assert 等の例題の補助クラスです。
    /// その他、各例題共通項目のためのクラス(全て static)。<br/>
    ///   double PI = 3.14159265358979      ... 円周率<br/>
    ///   int    Frac = 3;                  ... 小数点以下の出力桁数（デフォルトは３）<br/>
    ///   string Format = "F"+Frac;         ... 数値を出力する書式（デフォルトは　"F3"）<br/>
    ///   int    Length = Frac+3;           ... 数値出力の全桁数（デフォルトは　Frac+3）<br/>
    ///   void   P(string st)               ... 文字列(st)を出力後、改行します。<br/>
    ///   void   P(string title, _Matrix m) ... 行列(m)を出力<br/>
    ///   void   P(string title, _Vector v) ... ベクトル(v)を出力<br/>
    ///   void   W(string st)               ... st をコンソールに出力（改行無）。<br/>
    ///   void   WL(string st)              ... st をコンソールに出力（改行有）。<br/>
    ///   string R(string st)               ... 文字列を右詰めに(前にスペースを付けてから後ろからLength個の文字をリターン)する。<br/>
    ///   void   Assert(bool f)             ... f が true か確認します。true なら何もしません。false ならメッセージボックスを表示します。<br/>
    ///   void   Assert(bool f, string st)  ... f が true か確認します。true なら何もしません。false ならタイトル文字のメッセージボックスを表示します。<br/>
    ///   void   EQ(t1, t2)                 ... t1 と t2 が等しいかチェックします。等しいなら何もしません。等しくなければメッセージボックスを表示します。<br/>
    //                                          t1/t2 は double/Complex/_Vector/_Matrix を指定可。<br/>
    /// </summary>
    public static class T
    {
        /// <summary>
        /// 円周率
        /// </summary>
        public  const double PI = 3.14159265358979;

        /// <summary>
        /// 小数点以下の出力桁数（デフォルトは３）
        /// </summary>
        public static int    Frac = 3;
        /// <summary>
        /// 数値を出力する書式（デフォルトは　"F3"）
        /// </summary>
        public static string Format = "F"+Frac;
        /// <summary>
        /// 数値出力する場合の先頭空白
        /// </summary>
        public static string Lead   = "          ";
        /// <summary>
        /// 数値出力の全桁数（デフォルトは　Frac+3）
        /// </summary>
        public static int    Length = Frac+4; // ' -0.' の 4

        /// <summary>
        /// 文字列を出力後、改行します。
        /// </summary>
        /// <param name="st">出力する文字列</param>
        public static void P(string st)
        {
            WL("\n"+st);
        }

        /// <summary>
        /// 行列を出力
        /// </summary>
        /// <param name="title">表題</param>
        /// <param name="m">出力する行列</param>
        public static void P(string title, _Matrix m)
        {
            WL("\n" + title);
            if (m is _MatrixDouble)
            {
                _MatrixDouble md = (_MatrixDouble)m;
                for (int i = 1; i <= m.RowCount; ++i)
                {
                    for (int j = 1; j <= m.ColumnCount; ++j)
                    {
                        W(" " + R(md[i, j].ToString(Format)));
                    }
                    WL("");
                }
            }
            else
            {
                _MatrixComplex mc = (_MatrixComplex)m;
                for (int i = 1; i <= m.RowCount; ++i)
                {
                    for (int j = 1; j <= m.ColumnCount; ++j)
                    {
                        W(" (" + R(mc[i, j].Real.ToString(Format)) + "," + R(mc[i, j].Imaginary.ToString(Format)) + ")");
                    }
                    WL("");
                }
            }
        }

        /// <summary>
        /// ベクトルを出力
        /// </summary>
        /// <param name="title">表題</param>
        /// <param name="v">出力するベクトル</param>
        public static void P(string title, _Vector v)
        {
            WL("\n" + title);
            if (v is _VectorDouble)
            {
                _VectorDouble vd = (_VectorDouble)v;
                for (int i = 1; i <= v.Size; ++i)
                {
                    W(" " + R(vd[i].ToString(Format)));
                }
            }
            else if (v is _VectorComplex)
            {
                _VectorComplex vc = (_VectorComplex)v;
                for (int i = 1; i <= v.Size; ++i)
                {
                    W(" (" + R(vc[i].Real.ToString(Format)) + "," + R(vc[i].Imaginary.ToString(Format)) + ")");
                }
            }
            else
            {
                VectorDenseInt vi = (VectorDenseInt)v;
                for (int i = 1; i <= v.Size; ++i)
                {
                    W(" " + R(vi[i].ToString()));
                }
            }
            WL("");
        }

        /// <summary>
        /// コンソール出力（改行無）。
        /// </summary>
        /// <param name="st">出力文字列</param>
        public static void W(string st)
        {
            Console.Write(st);
        }

        /// <summary>
        /// コンソール出力（改行有り）。
        /// </summary>
        /// <param name="st">出力文字列</param>
        public static void WL(string st)
        {
            Console.WriteLine(st);
        }

        /// <summary>
        /// 文字列を右詰めに(前にスペースを付けてから後ろからLength個の文字をリターン)する。
        /// </summary>
        /// <param name="st">右詰にする文字列</param>
        /// <returns>右詰文字列</returns>
        public static string R(string st)
        {
            int ix = Lead.Length + st.Length - Length - 1;
            if (ix < 0) ix = 0;
            if (ix>=Lead.Length) ix = Lead.Length-1;
            return (Lead + st).Substring(ix);
        }

        /// <summary>
        /// 数値を文字列にして右詰めに(前にスペースを付けてから後ろからLength個の文字をリターン)する。
        /// </summary>
        /// <param name="v">右詰に表示する数値列</param>
        /// <returns>右詰文字列</returns>
        public static string R(double v)
        {
            string st = v.ToString(Format);
            int ix = Lead.Length + st.Length - Length - 1;
            if (ix < 0) ix = 0;
            if (ix >= Lead.Length) ix = Lead.Length - 1;
            return (Lead + st).Substring(ix);
        }

        /// <summary>
        /// f が true か確認します。true なら何もしません。false ならメッセージボックスを表示します。
        /// </summary>
        /// <param name="f">チェック引数</param>
        public static void Assert(bool f)
        {
            if (f) return;
            StackFrame CallStack = new StackFrame(1, true);
            string SourceFile = CallStack.GetFileName();
            int SourceLine = CallStack.GetFileLineNumber();
            MessageBox.Show("Assertion failed"+ " File: " + SourceFile + " Line: " + SourceLine.ToString());

        }

        /// <summary>
        /// f が true か確認します。true なら何もしません。false ならタイトル文字のメッセージボックスを表示します。
        /// </summary>
        /// <param name="f">チェック引数</param>
        /// <param name="st">タイトル文字</param>
        public static void Assert(bool f, string st)
        {
            if (f) return;
            StackFrame CallStack = new StackFrame(2, true);
            string SourceFile = CallStack.GetFileName();
            int SourceLine = CallStack.GetFileLineNumber();
            MessageBox.Show("Assertion failed(" +st + ") " + " File: " + SourceFile + " Line: " + SourceLine.ToString());
        }

        /// <summary>
        /// c1 と c2 が等しいかチェックします。等しいなら何もしません。等しくなければメッセージボックスを表示します。
        /// </summary>
        /// <param name="c1">第一変数</param>
        /// <param name="c2">第二変数</param>
        public static void EQ(Complex c1, Complex c2)
        {
            if (_Mol.EQ(c1, c2)) return;
            T.Assert(false, c1.ToString() + "!=" + c2.ToString());
        }

        /// <summary>
        /// c1 と c2 が等しいかチェックします。等しいなら何もしません。等しくなければメッセージボックスを表示します。
        /// </summary>
        /// <param name="c1">第一変数</param>
        /// <param name="c2">第二変数</param>
        /// <param name="eps">誤差判定定数</param>
        public static void EQ(Complex c1, Complex c2,double eps)
        {
            if (_Mol.EQ(c1, c2, eps)) return;
            T.Assert(false, c1.ToString() + "!=" + c2.ToString());
        }

        
        /// <summary>
        /// v1 と v2 が等しいかチェックします。等しいなら何もしません。等しくなければメッセージボックスを表示します。
        /// </summary>
        /// <param name="v1">第一変数</param>
        /// <param name="v2">第二変数</param>
        public static void EQ(double v1, double v2)
        {
            if (_Mol.EQ(v1, v2)) return;
            T.Assert(false, v1.ToString() + "!=" + v2.ToString());
        }

        /// <summary>
        /// v1 と v2 が等しいかチェックします。等しいなら何もしません。等しくなければメッセージボックスを表示します。
        /// </summary>
        /// <param name="v1">第一変数</param>
        /// <param name="v2">第二変数</param>
        /// <param name="eps">誤差判定定数</param>
        public static void EQ(double v1, double v2, double eps)
        {
            if (_Mol.EQ(v1, v2, eps)) return;
            T.Assert(false, v1.ToString() + "!=" + v2.ToString());
        }

        /// <summary>
        /// v1 と v2 が等しいかチェックします。等しいなら何もしません。等しくなければメッセージボックスを表示します。
        /// </summary>
        /// <param name="v1">第一変数</param>
        /// <param name="v2">第二変数</param>
        public static void EQ(_Vector v1, _Vector v2)
        {
            if (_Mol.EQ(v1, v2)) return;
            T.Assert(false, v1.ToString() + "!=" + v2.ToString());
        }

        /// <summary>
        /// v1 と v2 が等しいかチェックします。等しいなら何もしません。等しくなければメッセージボックスを表示します。
        /// </summary>
        /// <param name="v1">第一変数</param>
        /// <param name="v2">第二変数</param>
        /// <param name="eps">誤差判定定数</param>
        public static void EQ(_Vector v1, _Vector v2, double eps)
        {
            if (_Mol.EQ(v1, v2, eps)) return;
            T.Assert(false, v1.ToString() + "!=" + v2.ToString());
        }

        /// <summary>
        /// m1 と m2 が等しいかチェックします。等しいなら何もしません。等しくなければメッセージボックスを表示します。
        /// </summary>
        /// <param name="m1">第一変数</param>
        /// <param name="m2">第二変数</param>
        public static void EQ(_Matrix m1, _Matrix m2)
        {
            if (_Mol.EQ(m1, m2)) return;
            T.Assert(false, m1.ToString() + "!=" + m2.ToString());
        }

        /// <summary>
        /// m1 と m2 が等しいかチェックします。等しいなら何もしません。等しくなければメッセージボックスを表示します。
        /// </summary>
        /// <param name="m1">第一変数</param>
        /// <param name="m2">第二変数</param>
        /// <param name="eps">誤差判定定数</param>
        public static void EQ(_Matrix m1, _Matrix m2, double eps)
        {
            if (_Mol.EQ(m1, m2, eps)) return;
            T.Assert(false, m1.ToString() + "!=" + m2.ToString());
        }
    }
}
