﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Diagnostics;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using Mol;
using BigExamples;

namespace Examples
{
    public class TestBigDecimal
    {
        public static void Test()
        {
            {
                BigDecimal b = BigDecimal.Three.Sqrt(100); // 100桁
                T.P("Sqrt(3)    =" + b);
                T.P("Sqrt(3)**2 =" + (b*b));
                T.P("Sqrt(3).LengthRound(20, BigDecimal.ROUND_MODE.DOWN)     =" + b.LengthRound(20, BigDecimal.ROUND_MODE.DOWN));
                T.P("Sqrt(3).LengthRound(20, BigDecimal.ROUND_MODE.UP)       =" + b.LengthRound(20, BigDecimal.ROUND_MODE.UP));
                T.P("Sqrt(3).LengthRound(10, BigDecimal.ROUND_MODE.HALF_UP)  =" + b.LengthRound(10, BigDecimal.ROUND_MODE.HALF_UP));
                T.P("Sqrt(3).LengthRound(10, BigDecimal.ROUND_MODE.HALF_DOWN)=" + b.LengthRound(10, BigDecimal.ROUND_MODE.HALF_DOWN));
                T.P("Sqrt(3).PointRound(10, BigDecimal.ROUND_MODE.CEIL)      =" + b.PointRound(10, BigDecimal.ROUND_MODE.CEIL));
                T.P("Sqrt(3).PointRound(10, BigDecimal.ROUND_MODE.FLOOR)     =" + b.PointRound(10, BigDecimal.ROUND_MODE.FLOOR));
                b = b.LengthRound(30);
                T.P("b.ToString(\"+F,10\")="+b.ToString("+F,10"));
                T.P("b.ToString(\" F\")=" + b.ToString(" F"));
            }

            { // 連立方程式
                int n = 10;
                BigDecimal a = new BigDecimal("3");
                BigDecimal[,] A = new BigDecimal[n, n];
                BigDecimal[,] AS = new BigDecimal[n, n];
                BigDecimal[] b = new BigDecimal[n];

                // 要素値のセット
                // 対角要素のみ３、両端が１の三重対角行列
                for (int i = 0; i < n; ++i)
                {
                    if (i - 1 >= 0) A[i - 1, i] = BigDecimal.One;
                    A[i, i] = a.Clone();
                    if (i + 1 < n) A[i, i + 1] = BigDecimal.One;
                }
                for (int i = 0; i < n; i += 2) b[i] = new BigDecimal("" + i);

                //
                // A は LU に置き換えられるので元の値を AS に保存。
                for (int i = 0; i < n; ++i)
                {
                    for (int j = 0; j < n; ++j)
                    {
                        if (A[i, j] == null) AS[i, j] = BigDecimal.Zero;
                        else AS[i, j] = A[i, j].Clone();
                    }
                }

                // LU 因子分解
                int[] ps = BigMath.LuDecomp(A, 50);
                // 解の計算
                BigDecimal[] x = BigMath.LuSolve(A, b, ps, 50);

                // 検算
                // null をゼロに置き換える
                for (int i = 0; i < n; ++i)
                {
                    if (b[i] == null) b[i] = BigDecimal.Zero;
                }
                for (int i = 0; i < n; ++i)
                {
                    T.WL("x[" + i + "]=" + x[i]);
                }
                for (int i = 0; i < n; ++i)
                {
                    BigDecimal s = BigDecimal.Zero;
                    for (int j = 0; j < n; ++j)
                    {
                        s += AS[i, j] * x[j];
                    }
                    T.WL("R[" + i + "]=" + (s - b[i]));
                }
            }
            {
                T.WL("PI(100)=" + BigMath.PI(100).ToString());

                BigDecimal x = new BigDecimal("100");
                BigDecimal sinx = BigMath.Sin(x, 100);
                BigDecimal cosx = BigMath.Cos(x, 100);
                T.WL("Sin(100)=" + sinx.ToString());
                T.WL("Cos(100)=" + cosx.ToString());
                T.WL("Cos**2 + Sin**2=" + (cosx * cosx + sinx * sinx).ToString());

                BigDecimal e = BigMath.E(100);
                T.WL("E(100)=" + e.ToString());
                BigDecimal z = BigMath.Log(e, 100);
                T.WL("Log(e)=" + z.ToString());
            }
        }
    }
}
