﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Diagnostics;
using System.Drawing;
using System.Linq;
using System.Numerics;
using System.Text;
using System.Windows.Forms;

using Mol;

namespace Examples
{
    /// <summary>
    /// 固有値問題のテスト（例題）クラス
    /// </summary>
    public class TestEgv
    {
        /// <summary>
        /// 固有値問題の例題を実行します。
        /// </summary>
        public static void Test()
        {
            { // See: http://software.intel.com/sites/products/documentation/hpc/mkl/lapack/mkl_lapack_examples/dsyevx_ex.c.htm
                MatrixDenseSymmetricDouble S = new MatrixDenseSymmetricDouble(5);
                S[1, 1] = 6.29; S[1, 2] = -0.39; S[1, 3] = 0.61; S[1, 4] = 1.18; S[1, 5] = -0.08;
                S[2, 2] = 7.19; S[2, 3] = 0.81; S[2, 4] = 1.19; S[2, 5] = -0.08;
                S[3, 3] = 5.48; S[3, 4] = -3.13; S[3, 5] = 0.22;
                S[4, 4] = 3.79; S[4, 5] = -0.26;
                S[5, 5] = 0.83;

                MatrixDenseGeneralDouble A = new MatrixDenseGeneralDouble(5, 5);
                for (int i = 1; i <= 5; ++i)
                {
                    for (int j = 1; j <= 5; ++j) A[i, j] = S[i, j];
                }

                VectorDenseDouble eigen_values = new VectorDenseDouble(5);
                MatrixDenseGeneralDouble eigen_vectors = new MatrixDenseGeneralDouble(5, 5);

                T.P("General A:", A);
                EgvSolver egv = EgvSolver.Solve(eigen_values, eigen_vectors, (MatrixDenseGeneralDouble)A.Clone());
                T.P("eigen_values:", eigen_values);
                T.P("eigen_vectors:", eigen_vectors);
                for (int i = 1; i <= 5; ++i)
                {
                    T.P("A*eigen_vectors[" + i + "] - eigen_values[" + i + "]*eigen_vectors[" + i + "]", A * eigen_vectors.Columns[i] - eigen_values[i] * eigen_vectors.Columns[i]);
                }

                T.P("Symmetric S:", S);
                egv = EgvSolver.Solve(eigen_values, eigen_vectors, (MatrixDenseSymmetricDouble)S.Clone());
                T.P("eigen_values:", eigen_values);
                T.P("eigen_vectors:", eigen_vectors);
                for (int i = 1; i <= 5; ++i)
                {
                    T.P("S*eigen_vectors[" + i + "] - eigen_values[" + i + "]*eigen_vectors[" + i + "]", S * eigen_vectors.Columns[i] - eigen_values[i] * eigen_vectors.Columns[i]);
                }
            }
            { // 汎用問題（Bは正定値）
                MatrixDenseHermite A = new MatrixDenseHermite(4);
                A[1, 1] = new Complex(3, 0); A[1, 2] = new Complex(0, 2); A[1, 3] = new Complex(1, 1); A[1, 4] = new Complex(0, 1);
                A[2, 2] = new Complex(3, 0); A[2, 3] = new Complex(0, 2); A[2, 4] = new Complex(1, 1);
                A[3, 3] = new Complex(3, 0); A[3, 4] = new Complex(0, 2);
                A[4, 4] = new Complex(3, 0);

                MatrixDenseHermite B = new MatrixDenseHermite(4);
                B[1, 1] = new Complex(2, 0); B[1, 2] = new Complex(1, 1); B[1, 3] = new Complex(0, 1);
                B[2, 2] = new Complex(2, 0); B[2, 3] = new Complex(1, 1); B[2, 4] = new Complex(0, 1);
                B[3, 3] = new Complex(2, 0); B[3, 4] = new Complex(1, 1);
                B[4, 4] = new Complex(2, 0);

                VectorDenseComplex eigen_values = new VectorDenseComplex(4);
                MatrixDenseGeneralComplex eigen_vectors = new MatrixDenseGeneralComplex(4, 4);
                T.P("A:", A);
                T.P("B:", B);
                EgvSolver egv = EgvSolver.Solve2(eigen_values, eigen_vectors, (MatrixDenseHermite)A.Clone(), (MatrixDenseHermite)B.Clone());
                T.P("eigen_values:", eigen_values);
                T.P("eigen_vectors:", eigen_vectors);
                for (int j = 1; j <= 4; ++j)
                {
                    T.P("A*eigen_vectors.Columns[" + j + "] - eigen_values[" + j + "]*B*eigen_vectors.Columns[" + j + "] =", A * eigen_vectors.Columns[j] - eigen_values[j] * B * eigen_vectors.Columns[j]);
                }
            }

        }
    }
}
