﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Diagnostics;
using System.Drawing;
using System.Linq;
using System.Numerics;
using System.Text;
using System.Windows.Forms;

using Mol;

namespace Examples
{
    /// <summary>
    /// 行列のテスト（例題）クラス
    /// </summary>
    public class TestMatrix
    {
        /// <summary>
        /// 行列の例題を実行します。
        /// </summary>
        public static void Test()
        {
            MatrixDenseGeneralDouble  mdgd = new MatrixDenseGeneralDouble(3, 4);
            MatrixSparseGeneralDouble msgd = new MatrixSparseGeneralDouble(3, 4);
            for (int i = 1; i <= mdgd.RowCount; ++i)
            {
                for (int j = 1; j <= mdgd.ColumnCount; j+=2)
                {
                    mdgd[i, j] = i * 10 + j;
                    msgd[i, j] = i * 10 + j;
                }
            }
            T.P("mdgd", mdgd);
            T.P("mdgd+msgd", mdgd+msgd);
            T.P("mdgd.Transpose()", mdgd.Transpose());
            T.P("mdgd==msgd: " +_Mol.EQ(mdgd,msgd));
            T.P("mdgd.Count=" + mdgd.Count);
            T.P("msgd.Count=" + msgd.Count);
            T.P("mdgd*2=" , mdgd*2.0);
            T.P("msgd/2=" , msgd/2.0);
            T.P("msgd.Norm1()=" + msgd.Norm1());
            T.P("msgd.NormE()=" + msgd.NormE());
            T.P("msgd.NormI()=" + msgd.NormI());

            MatrixDenseHermite  mdh = new MatrixDenseHermite(4);
            MatrixSparseHermite msh = new MatrixSparseHermite(4);
            mdh[1, 1] = new Complex(1, 0); mdh[1, 2] = new Complex(1, 2); mdh[1, 4] = new Complex(1, 4);
            mdh[2, 2] = new Complex(2, 0); mdh[2, 3] = new Complex(2, 3);
            mdh[3, 3] = new Complex(3, 0); mdh[3, 4] = new Complex(3, 4);
            mdh[4, 4] = new Complex(4, 0);
            // 非ゼロ要素を msh に移す。
            for (int i = 1; i <= mdh.RowCount; ++i)
            {
                for (int j = i; j <= mdh.ColumnCount; ++j)
                {
                    if (Complex.Abs(mdh[i, j]) <= _Mol.EPS) continue;
                    msh[i, j] = mdh[i, j];
                }
            }
            T.P("msh:", msh);
            T.P("msh==mdh: " + _Mol.EQ(msh, mdh));
            for (int i = 1; i <= msh.RowCount; ++i)
            {
                T.P("msh.Rows[" + i + "]", msh.Rows[i]);
            }
            for (int i = 1; i <= msh.ColumnCount; ++i)
            {
                T.P("mdh.Columns[" + i + "]", mdh.Columns[i]);
            }
            T.P("msh==mdh.TransposeConjugate(): " + _Mol.EQ(msh,mdh.TransposeConjugate()));
            //
            // Band 行列
            MatrixDenseBandDouble mdbd = new MatrixDenseBandDouble(5, 4, 2, 1);
            mdbd[1, 1] = 11; mdbd[1, 2] = 12; mdbd[1, 3] = 13;
            mdbd[2, 1] = 21; mdbd[2, 2] = 22; mdbd[2, 3] = 23; mdbd[2, 4] = 24;
                             mdbd[3, 2] = 32; mdbd[3, 3] = 33; mdbd[3, 4] = 34;
                                              mdbd[4, 3] = 43; mdbd[4, 4] = 44;
            T.P("mdbd:",mdbd);
            for (int i = 1; i <= mdbd.RowCount; ++i)
            {
                T.W("\n[" + i + "]");
                int nj = mdbd.GetRegisteredElementCount(i);
                for (int jx = 1; jx <= nj; ++jx)
                {
                    INDEX_PAIR ix = mdbd.GetArrayIndex(i, jx);
                    Complex v = mdbd.GetElement(ix.ArrayIndex);
                    T.W(" " + ix.Index + ":" + v.Real); // 実数だから
                }
            }
            T.P("mdbd.Transpose():", mdbd.Transpose());
            //
            // 三角 行列
            MatrixDenseUpperTriangleDouble  mdutd = new MatrixDenseUpperTriangleDouble(4);
            MatrixSparseLowerTriangleDouble msltd = new MatrixSparseLowerTriangleDouble(4);
            for (int i = 1; i <= mdutd.RowCount; ++i)
            {
                for (int j = i; j <= mdutd.ColumnCount; ++j) mdutd[i, j] = i * 10 + j;
                for (int j = 1; j <i; ++j) msltd[i, j] = i * 10 + j;
            }
            T.P("mdutd", mdutd);
            T.P("msltd", msltd);
            T.P("mdutd+msltd", mdutd + msltd);
            T.P("mdutd.Transpose()", mdutd.Transpose());
            T.P("msltd.Transpose()", msltd.Transpose());
            T.P("mdutd.Transpose()+msltd.Transpose()", mdutd.Transpose() + msltd.Transpose());
        }
    }
}
