﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Diagnostics;
using System.Drawing;
using System.Linq;
using System.Numerics;
using System.Text;
using System.Windows.Forms;

using Mol;

namespace Examples
{
    /// <summary>
    /// 特異値分解のテスト（例題）クラス
    /// </summary>
    public class TestSvd
    {
        /// <summary>
        /// 特異値分解の例題を実行します。
        /// </summary>
        public static void Test()
        {
            {
                MatrixDenseGeneralDouble A = new MatrixDenseGeneralDouble(3, 3);
                A[1, 1] = 7; A[1, 2] = 2;
                A[2, 1] = 2; A[2, 2] = 6; A[2, 3] = -2;
                A[3, 2] = -2; A[3, 3] = 5;

                MatrixDenseGeneralDouble U = new Mol.MatrixDenseGeneralDouble(3, 3);
                MatrixDenseGeneralDouble V = new Mol.MatrixDenseGeneralDouble(3, 3);
                MatrixDenseBandDouble s = SvdSolver.Solve((MatrixDenseGeneralDouble)A.Clone(), null, U, V);
                // s[1] = 9; s[2] = 6; s[3] = 3;
                T.P("A", A);
                T.P("U", U);
                T.P("s", s);
                T.P("V", V);

                T.P("(A, U * s * V).NormE()=" + (A - U * s * V).NormE());

                MatrixDenseGeneralDouble VT = V.Transpose();
                MatrixDenseGeneralDouble UT = U.Transpose();

                T.P("U*UT", U * U.Transpose());
                T.P("V*VT", V * V.Transpose());

                // 以下は同じこと…
                for (int i = 1; i <= 3; ++i)
                {
                    T.P("A * VT.Columns[" + i + "] - s[" + i + "," + i + "] * U.Columns[" + i + "]:", A * VT.Columns[i] - s[i, i] * U.Columns[i]);
                }
                for (int i = 1; i <= 3; ++i)
                {
                    T.P("UT.Rows[" + i + "] * A - s[" + i + "," + i + "] * V.Rows[" + i + "]:", UT.Rows[i] * A - s[i, i] * V.Rows[i]);
                }
            }

            {
                // See:  http://software.intel.com/sites/products/documentation/hpc/mkl/lapack/mkl_lapack_examples/zgesdd_ex.c.htm
                //
                MatrixDenseGeneralComplex A = new MatrixDenseGeneralComplex(3, 4);
                A[1, 1] = new Complex(-5.40, 7.40); A[1, 2] = new Complex(6.00, 6.38); A[1, 3] = new Complex(9.91, 0.16); A[1, 4] = new Complex(-5.28, -4.16);
                A[2, 1] = new Complex(1.09, 1.55); A[2, 2] = new Complex(2.60, 0.07); A[2, 3] = new Complex(3.98, -5.26); A[2, 4] = new Complex(2.03, 1.11);
                A[3, 1] = new Complex(9.88, 1.91); A[3, 2] = new Complex(4.92, 6.31); A[3, 3] = new Complex(-2.11, 7.39); A[3, 4] = new Complex(-9.81, -8.98);

                T.P("A:", A);
                MatrixDenseGeneralComplex U = new Mol.MatrixDenseGeneralComplex(3, 3);
                MatrixDenseGeneralComplex V = new Mol.MatrixDenseGeneralComplex(4, 4);
                MatrixDenseBandComplex s = SvdSolver.Solve2((MatrixDenseGeneralComplex)A.Clone(), null, U, V);
                T.P("U:", U);
                T.P("s:", s);
                T.P("V(Conjugate transposed):", V);

                T.P("(A, U * s * V).NormE()=" + (A - U * s * V).NormE());

                MatrixDenseGeneralComplex VT = V.TransposeConjugate();
                MatrixDenseGeneralComplex UT = U.TransposeConjugate();

                T.P("U*U.TransposeConjugate()", U * UT);
                T.P("V*V.TransposeConjugate()", VT * V);

                // 以下は同じこと…
                for (int i = 1; i <= 3; ++i)
                {
                    T.P("A * VT.Columns["+i+"] - s["+i+","+ i+"] * U.Columns["+i+"]:", A * VT.Columns[i] - s[i, i] * U.Columns[i]);
                }
                for (int i = 1; i <= 3; ++i)
                {
                    T.P("UT.Rows["+i+"] * A - s[" + i + "," + i + "] * V.Rows[" + i + "]:", UT.Rows[i]*A  - s[i, i] * V.Rows[i]);
                }
            }
        }
    }
}
